from flask import Flask, request, render_template, Response
import requests
from bs4 import BeautifulSoup
import pandas as pd
import io
import re

# Define dfTemp as a global variable
dfTemp = None

def get_keyword_traffic(search_keyword, num_pages):
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"
    }
    keyword_traffic = []
    for page in range(num_pages):
        start_index = page * 10
        query = f"{search_keyword}&start={start_index}"
        google_search_url = f"https://www.google.com/search?q={query}"
        response = requests.get(google_search_url, headers=headers)
        soup = BeautifulSoup(response.text, "html.parser")
        search_results = soup.find_all("div", class_="tF2Cxc")
        for result in search_results:
            title_element = result.find("h3")
            keyword = title_element.get_text()
            url_element = result.find("a")
            traffic_source = url_element["href"]
            keyword_traffic.append({"Keyword": keyword, "Traffic Source": traffic_source})
    return keyword_traffic

app = Flask(__name__)

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/result', methods=['POST'])
def result():
    global dfTemp  # Use global to access the global variable
    search_keyword = request.form['check']
    num_pages = int(request.form['check2'])
    df = pd.DataFrame(get_keyword_traffic(search_keyword, num_pages))
    dfTemp = df  # Update the global variable
    df['Traffic Source'] = df['Traffic Source'].apply(lambda x: f'<a href="{x}" target="_blank">Open Link</a>')
    return render_template('result.html', table=df.to_html(escape=False, classes='table table-striped'))

@app.route('/download_csv', methods=['POST'])
def download_csv():
    global dfTemp  # Use global to access the global variable
    df = dfTemp
    df['Traffic Source'] = df['Traffic Source'].apply(lambda x: re.search(r'href="(.*?)"', x).group(1) if re.search(r'href="(.*?)"', x) else '')
    # Convert the DataFrame to a CSV string
    csv_data = df.to_csv(index=False, encoding='utf-8-sig')
      # Create a Flask Response with the CSV data
    response = Response(csv_data, mimetype='text/csv')
    response.headers["Content-Disposition"] = "attachment; filename=searchResults.csv"
    return response

@app.route('/download_excel', methods=['POST'])
def download_excel():
    global dfTemp  # Use global to access the global variable
    df = dfTemp
    df['Traffic Source'] = df['Traffic Source'].apply(lambda x: re.search(r'href="(.*?)"', x).group(1) if re.search(r'href="(.*?)"', x) else '')
    # Create an Excel writer object
    excel_output = io.BytesIO()
    with pd.ExcelWriter(excel_output, engine='xlsxwriter') as writer:
        df.to_excel(writer, index=False, sheet_name='Sheet1')
    excel_output.seek(0)
    # Create a Flask Response with the Excel data
    response = Response(excel_output.read(), mimetype='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')
    response.headers["Content-Disposition"] = "attachment; filename=searchResults.xlsx"
    return response

if __name__ == '__main__':
    app.run(debug=True)
